/**********************************************************
 * CSE142, 99 Winter                                      *
 * Homework 5                                             *
 * Valentine's Day Card                                   *
 *                                                        *
 * Author:     Grant Yuan                                 *
 * Student ID: 9827015                                    *
 * Section:    AA                                         *
 **********************************************************/

#include "gp142.h"
#include <math.h>
#include <time.h>

// Arrays
#define ARRAYSIZE	200

// GP142 Defs
#define FILL			0
#define NOFILL			1

// Boolean
#define TRUE  1
#define FALSE 0
#define NOTHING -1

// Graphics defs
#define BUTTON_HEIGHT	25
#define BUTTON_WIDTH	100
#define BUTTON_XPAD		10
#define BUTTON_YPAD		20
#define ERROR_MARGIN	10
#define CARD_WARP		20
#define CARD_ANGLE_INCREASE 10

// Mental Status
#define NORMAL	1
#define HIT		2
#define TRIP	3

// Shapes
#define HEART	1
#define MESSAGE	2
#define PEACE	3
#define SMILEY	4

#define HEART_INCREMENT		4
#define	HEART_RADIUS		20
#define	HEART_HEIGHT		(int) (HEART_RADIUS * 1.25)
#define HEART_OUTLINE		BLACK
#define DEFAULT_HEART_COLOR	RED

#define SMILEY_WIDTH			30
#define DEFAULT_SMILEY_COLOR	YELLOW
#define TIME_TO_BLINK			15

#define PEACE_INNER_WIDTH		15
#define PEACE_OUTER_WIDTH		20
#define PEACE_BAR_WIDTH			3
#define PEACE_RUN_INCREMENT		5
#define DEFAULT_PEACE_COLOR		GREEN

#define DEFAULT_MSG_COLOR		CHALK
#define MESSAGE_SIZE			14

// Math
#define PI 3.14
#define COS225 -0.707107
#define SIN225 -0.707107
#define COS315 0.707107
#define SIN315 -0.707107

// Ok, the default constants hurt my eyes...
#define XMIN -GP142_XMAX
#define XMAX GP142_XMAX
#define YMIN -GP142_YMAX
#define YMAX GP142_YMAX
#define CARD_X1 (XMIN + BUTTON_WIDTH +60)
#define CARD_Y1 (YMIN + 100)
#define CARD_WIDTH (XMAX - CARD_X1 - 80)
#define CARD_HEIGHT (YMAX - CARD_Y1 - 40)
#define CARD_X2 (CARD_X1 + CARD_WIDTH)
#define CARD_Y2 (CARD_Y1 + CARD_HEIGHT)

// Current Status
#define SEL_NONE		0
#define SEL_BACKGROUND	1
#define SEL_HEART		2
#define SEL_SMILEY		3
#define SEL_PEACE		4
#define SEL_MESSAGE		5
#define SEL_OBJECT		6
#define SEL_COLOR		7
#define SEL_HIT			8
#define SEL_TRIP		9
#define SEL_WISDOM		10
#define SEL_CLEAR		11
#define SEL_CARD		12
#define MAX_BUTTONS		11


// *** Prototypes *** //
void CardIntro(int *x, int *y, char *key, int *quit);
void DrawBlankScreen (void); 
void DrawButtons(int ButtonStatus);
void DrawCard (int BGColor, int ShapeArray[ARRAYSIZE][5], int numShapes, 
			   int ButtonStatus, int MentalStatus, int * ment_angle);
void DrawCrazyScreen(int * hit_bg);
void DrawHeart (int x, int y, int color, int *frame, int BGColor);
void DrawMessage(int x, int y, int color, int *frame);
void DrawPeace(int x, int y, int color, int *frame, int BGColor);
void DrawSmiley(int x, int y, int color, int *frame);
void Ending(int *x, int *y, char *key_hit);
void FindObject(int x, int y, int ShapeArray[ARRAYSIZE][5], int numshapes, 
				int *currentShape, int *hilight);
void HandleAction(int *x, int *y, int ButtonStatus, int *hilight, 
				  int ShapeArray[ARRAYSIZE][5], int *numShapes, int *BGColor, 
				  int *MentalStatus, int * currentShape, char *heyhit);
void Intro (int *x, int *y, char *key_hit);
void TripOutTime(int ShapeArray[ARRAYSIZE][5], int numShapes, int *mouse_x, 
				 int *mouse_y, char *key_hit, int *quit, int *mental_status);
void Wisdom(int *x, int *y, char *key_hit);
int GetColor (int color);
int MouseClick(int mouse_x, int mouse_y);


// *** Main *** //
int main(void) {

	int BGColor;			// Background color
	int ButtonStatus;		// Which button is active
	int event;				// Recemt GP142 Event
	int hilight;			// Which button to highlight
	int MentalStatus;		// State of mind of user
	int mouse_x, mouse_y;	// XY Pos
	int numShapes;			// Total Number of Shapes	
	int SelectedShape;		// Currently selected object
	int quit;				// If we should quit program
	char key_hit;			// Null

	// Hit Variables
	int hit_bg;				// Hit background color
	int ment_angle;			// Warping angle

	// Shape array (x,y,color,frame,type)
	int ShapeArray[ARRAYSIZE][5];	


	// Initializations
	quit = FALSE;			// Keep running until true
	ButtonStatus = SEL_NONE;// Nothing selected
	hilight = SEL_NONE;		// Nothing highlighted
	numShapes = 0;			// Total number of shapes
	BGColor = PINK;			// Default Background
	SelectedShape = NOTHING;// Currently Selected Shape
	MentalStatus = NORMAL;	// Defaults to normal operation
	hit_bg = RED;			// Default hit background starter color
	ment_angle = 0;			// Mental angle

	// Randomizer stuff
	srand((unsigned) time(NULL));

	// GP142
	GP142_open();           // Open and init the GP142 Graphics Window      
	GP142_show_text(0);     // Hide the text window                         
	GP142_logging(LOG_OFF); // Change to LOG_ON if you want it.             
	GP142_clear();	        // This will clear the window & paint it white. 
	GP142_animate(ANI_RUN); // Enable periodic events                       


	// Call introduction sequence
	Intro (&mouse_x, &mouse_y, &key_hit);

	// ** Main Loop ** 

	while (!quit) {

		// Depending on how the user is feeling, draw appropriately
		switch(MentalStatus) {
			
			case NORMAL:		// Redraw All Items
				DrawBlankScreen ();
				DrawCard (BGColor, ShapeArray, numShapes, ButtonStatus, 
					MentalStatus, &ment_angle);
				DrawButtons(hilight);
				break;

			case HIT:
				DrawCrazyScreen(&hit_bg);
				DrawButtons(hilight);
				DrawCard (BGColor, ShapeArray, numShapes, ButtonStatus, 
					MentalStatus, &ment_angle);
				break;

			case TRIP:
				TripOutTime(ShapeArray, numShapes, &mouse_x, &mouse_y, 
					&key_hit, &quit, &MentalStatus);
				break;

			default: // Shouldn't ever happen
				break;
		}

		// Wait for event
		event = GP142_await_event(&mouse_x, &mouse_y, &key_hit);

		// Figure out what to do
		switch (event) {

			case GP142_QUIT:
				quit = TRUE;	// Set flag to break loop
				break;
			
			case GP142_KBD:		// Keypress
				// Only time a keypress will ever be needed is if the
				// drawing on screen is taking up the whole space,  
				// like during the altered states, and user needs
				// to break out of it.
				MentalStatus = NORMAL;
				break;
			
			case GP142_MOUSE:	// Mouse Click
				ButtonStatus = MouseClick(mouse_x, mouse_y);
				// Keeps only "Shape" buttons highlighted. Clicking on 
				// other buttons that don't highlight won't cause the 
				// highlight to turn off.
				if (ButtonStatus != SEL_COLOR && ButtonStatus != SEL_HIT && 
					ButtonStatus != SEL_TRIP  && ButtonStatus != SEL_CLEAR &&
					ButtonStatus != SEL_CARD  && ButtonStatus != SEL_NONE &&
					ButtonStatus != SEL_WISDOM) 
					hilight = ButtonStatus;

				HandleAction(&mouse_x, &mouse_y, ButtonStatus, &hilight, 
					ShapeArray, &numShapes, &BGColor, &MentalStatus, 
					&SelectedShape, &key_hit);
				break;
			
			case GP142_PERIODIC:
				/*An events has occurred simply because some time has passed,*/
				/*not currently used in the program*/		
				break;

			default:
				break;
			
		}  // END switch(event)
		
	}  // END while (!quit)
	
	// That's all Folks

	Ending(&mouse_x, &mouse_y, &key_hit);

	GP142_close();		// Clean Up
	return 0;


} // END Main()



// *** Functions *** //

void DrawBlankScreen (void) {
	GP142_rectangleXY (WHITE, XMIN, YMIN, XMAX, YMAX, FILL);
}


void DrawButtons(int ButtonStatus) {

	int t;			// Loop counter
	int x1, x2;		// X Coords for button
	int y1, y2;		// Y Coords for button
	int x_label;	// X Coords for label
	int y_label;	// Y Coords for label
	int numNull;	// Number of Null entries
	int fontsize;	// Font Size

	// Array for our button description values
	char bText[MAX_BUTTONS+2][25] = {
		"Background",
		"Heart",
		"Smiley Face",
		"Peace Sign",
		"Random Message",
		"Select Shape",
		"Change Color",
		"NULL",
		"Hit!",
		"Trip!",
		"Wisdom",
		"NULL",
		"Clear"};

	numNull = 2;
	fontsize = 12;

	// ButtonStatus is anything from 0 to 9, but the bText Array
	// is off by 1 compared to the array, but we want to keep it that
	// way, or it messes with the proceeding loop. So, offset the
	// ButtonStatus by one, and if it anything past the "Random Message"
	// button, make it null.
	if (ButtonStatus == SEL_NONE || ButtonStatus > SEL_COLOR) 
		ButtonStatus = 99; // Just some large number that t will never hit
	else
		ButtonStatus = ButtonStatus - 1;
		// Made sure if ButtonStatus = 0, it doesn't go here, since
		// we don't want negatives, even though the switch will be
		// fine anways..


	for(t=0; t < (MAX_BUTTONS + numNull) ; t++) {	
		x1 = XMIN + BUTTON_XPAD;
		x2 = x1 + BUTTON_WIDTH;
	
		// Make sure we keep adding y-offset since the buttons are
		// being drawn downward.
		y1 = (YMAX - BUTTON_YPAD) - (BUTTON_HEIGHT * t);
		y2 = y1 - BUTTON_HEIGHT;


		// If the array val is NULL, skip space
		if (strcmp(bText[t], "NULL") != 0) {

			// Draw solid light grey box, rimmed with darker outside, 
			// unless it's the currently selected button, then draw
			// it yellow instead
			if (ButtonStatus == t)
				GP142_rectangleXY(YELLOW, x1, y1, x2, y2, FILL);
			else
				GP142_rectangleXY(LT_GRAY, x1, y1, x2, y2, FILL);

			GP142_rectangleXY(MED_GRAY, x1, y1, x2, y2, NOFILL);

			// Draw our label in the box now
			x_label = x1 + 6;
			y_label = y1 - (int) (BUTTON_HEIGHT - 5);
			GP142_printfXY(BLUE, x_label, y_label, fontsize, bText[t]);
		}
	} // END for
} // END DrawButtons()


void DrawCard (int BGColor, int ShapeArray[ARRAYSIZE][5], int numShapes, 
			   int ButtonStatus, int MentalStatus, int *ment_angle)
{
	int i;			// Fraction for random movement
	int t;			// Loop Variable
	int x, y;		// Card coordinates

	// Draw Background
	if (MentalStatus == HIT) {
		x = (int) (cos((*ment_angle /180.0) * PI) * CARD_WARP);
		y = (int) (sin((*ment_angle /180.0) * PI) * CARD_WARP);

		// Draw card, with additional rotation offset
		GP142_rectangleXY (BGColor, CARD_X1 + x, CARD_Y1 + y, CARD_X2 + x, 
			CARD_Y2 + y, FILL);
		GP142_rectangleXY (BLACK, CARD_X1-1 + x, CARD_Y1-1 + y, 
			CARD_X2+1 + x, CARD_Y2+1 + y, NOFILL);

		// Add random offset to each array position
		for (t=0; t < numShapes; t++) {

				// Create a random number from -10 to 10, then
				// add it to the array x coordinate
				i = (rand() % 20) + 1;
				if (i >= 10) 
					i = i - 20;
				ShapeArray[t][0] = ShapeArray[t][0] + i;

				// Like before, but for y
				i = (rand() % 20) + 1;
				if (i >= 10) 
					i = i - 20;
				ShapeArray[t][1] = ShapeArray[t][1] + i;

				// If the shape goes off the screen, rewrap it
				// to the other side
				if (ShapeArray[t][0] >= (XMAX + 30))
					ShapeArray[t][0] = (XMIN - 30);
				if (ShapeArray[t][0] <= (XMIN - 30))
					ShapeArray[t][0] = (XMAX + 30);
				if (ShapeArray[t][1] >= (YMAX + 30))
					ShapeArray[t][1] = (YMIN - 30);
				if (ShapeArray[t][1] <= (YMIN - 30))
					ShapeArray[t][1] = (YMAX + 30);

				// Go to next color
				ShapeArray[t][2] = GetColor(ShapeArray[t][2]);

		}

		*ment_angle = *ment_angle + CARD_ANGLE_INCREASE;
		if (*ment_angle > 360) 
			*ment_angle = 0;
	} else {
		GP142_rectangleXY (BGColor, CARD_X1, CARD_Y1, CARD_X2, CARD_Y2, FILL);
		GP142_rectangleXY (BLACK, CARD_X1-1, CARD_Y1-1, CARD_X2+1, 
			CARD_Y2+1, NOFILL);
	}

	// Draw Shapes (Loop through all array objects)
	for (t=0; t < numShapes; t++) {

		// Determine type, then draw as appropriate
		switch (ShapeArray[t][4]) {
			case HEART:
				DrawHeart(ShapeArray[t][0], ShapeArray[t][1], ShapeArray[t][2], 
					&ShapeArray[t][3], BGColor);
				break;

			case MESSAGE:
				DrawMessage(ShapeArray[t][0], ShapeArray[t][1], 
					ShapeArray[t][2], &ShapeArray[t][3]);
				break;

			case PEACE:
				DrawPeace(ShapeArray[t][0], ShapeArray[t][1], 
					ShapeArray[t][2], &ShapeArray[t][3], BGColor);
				break;

			case SMILEY:
				DrawSmiley(ShapeArray[t][0], ShapeArray[t][1], 
					ShapeArray[t][2], &ShapeArray[t][3]);
				break;

			default:
				break;
		}
	} // END for
	
} // END DrawCard()

void DrawCrazyScreen(int * hit_bg) {

	int t;		// Loop var

	for(t=(int) (1.25 * XMAX); t > 10 ;) {

		*hit_bg = GetColor(*hit_bg);
		GP142_circleXY(*hit_bg, 0, 0, t);
		t = t - 20;
	}
}

void DrawHeart (int x, int y, int color, int *frame, int BGColor) {
// Will try to simulate a 3D rotating heart, by drawing two circles,
// one large, and one small, next to each other. The small circle
// becomes smaller, and the larger one becomes larger, after the
// smaller circle disappears, it pops up at the opposite side of
// the larger circle, and begins to grow in size. Meanwhile, a 
// triangle drawn between the two completes the heart shape. 

	int big_radius;	// Radius of bigger heart
	int sml_radius;	// Radius of little heart
	int heart_restart;	// Frame when heart repeats
	int t1x, t2x, t3x;	// Triangle X ccordinates
	int t1y, t2y, t3y;	// Triangle Y coordinates
	int x1, y1;			// Coordinates of big heart
	int x2, y2;			// Coordinates of small heart

	// Determine which frame will the frame begin to repeat
	heart_restart = HEART_RADIUS / HEART_INCREMENT;	

	// If the frame is greater than the Xth frame (when the heart
	// has completed rotation, make the radius of the bigger 
	// heart begin to decrease
	if (*frame <= heart_restart) 
		big_radius = *frame * HEART_INCREMENT;
	else
		big_radius = (heart_restart - *frame) * (-HEART_INCREMENT);

	sml_radius = HEART_RADIUS - big_radius;

	// Offsets from the center, so that the two centers
	// of each circle remain evenly rotated around the
	// normal x heart axis point.
	x1 = (int) (HEART_RADIUS - big_radius);
	x2 = (int) (HEART_RADIUS - sml_radius);

	y1 = y + 10;
	y2 = y + 10;

	if (*frame <= heart_restart) {
		x1 = x - x1;	// Big circle on the left side
		x2 = x + x2;	// Little circle on right side
	} 
	else {
		x1 = x + x1;	// Big circle on the right side
		x2 = x - x2;	// Little circle on left side
	}

	// Make heart outlines
	GP142_circleXY (HEART_OUTLINE, x1, y1, (int) big_radius);
	GP142_circleXY (HEART_OUTLINE, x2, y2, (int) sml_radius);

	// Fill in center of hearts

	if (big_radius == 0) big_radius = 1;
	if (sml_radius == 0) sml_radius = 1;

	GP142_circleXY (color, x1, y1, big_radius-1);
	GP142_circleXY (color, x2, y2, sml_radius-1);

	// Ok, now, we want to draw a good heart, so we'll take the
	// 225 degree angle of the left circle, and the 315 degree
	// angle of the right circle, and find the points there, then
	// make a triangle out of that. The angle values have been 
	// precalculated for efficiency. If it is past the restarting
	// frame, the circles will have switched, so get reverse points.

	if (*frame <= heart_restart) {
		t1x = (int) (COS225 * big_radius) + x1;
		t1y = (int) (SIN225 * big_radius) + y1;
		t2x = (int) (COS315 * sml_radius) + x2;
		t2y = (int) (SIN315 * sml_radius) + y2;
	} else {
		t1x = (int) (COS315 * big_radius) + x1;
		t1y = (int) (SIN315 * big_radius) + y1;
		t2x = (int) (COS225 * sml_radius) + x2;
		t2y = (int) (SIN225 * sml_radius) + y2;
	}

	t3x = x;
	t3y = y - HEART_HEIGHT;

	// Fill in triangle to complete heart
	GP142_triangleXY (color, t1x, t1y, t2x, t2y, t3x, t3y, FILL);

	// Draw outside of heart, because the large circles will
	// overlap the edges of the triangle and spill out of the heart.
	//GP142_triangleXY (BGColor, t1x, t1y, t1x, t3y, t3x, t3y, FILL);
	//GP142_triangleXY (BGColor, t2x, t2y, t2x, t3y, t3x, t3y, FILL);

	// Fix minor gfx problems with the middle of the heart
	GP142_lineXY(color, x-8, y+7, x+8, y+7, 5);

	// Outline heart with red lines
	GP142_lineXY (HEART_OUTLINE, t1x, t1y, t3x, t3y, 1);
	GP142_lineXY (HEART_OUTLINE, t2x, t2y, t3x, t3y, 1);

	*frame = *frame + 1;		// Increment the frame number

	if (*frame == (heart_restart * 2 + 1))	
		*frame = 1;		// Reset frame if it's gone through an entire loop

} // End DrawHeart()


void DrawMessage(int x, int y, int color, int *frame) {

	// Array of messages to print
	char message[7][25] = {
		"Groovy!",
		"Shagadelic!",
		"Way!",
		"Far Out!",
		"Flower Power!",
		"Damn Hippie!",
		"Yeah Babee Yeah!"
	};

	GP142_printfXY(BLACK, x+1, y-1, MESSAGE_SIZE, message[*frame]);
	GP142_printfXY(color, x, y, MESSAGE_SIZE, message[*frame]);

}


void DrawPeace(int x, int y, int color, int *frame, int BGColor) {
// Going to make a black peace emblem, with a user defined color traversing
// the outside of the circle in a pair. Starts at the top, and draws
// a line from the outer radius, to the inner radius, on the left and
// right side of the sign, so it's a mirrored effect. It gets drawn down
// ward, until it hits the bottom, then repeats.

	int t;					// Loop var
	int x1, y1;				// Coordinates for inside bars
	int x2, y2;
	int x3, y3;
	int left_x1,  left_y1;	// Left side band coordinates
	int left_x2,  left_y2;	// Left side, 2nd set
	int right_x1, right_y1;	// Right side, band coordinates
	int right_x2, right_y2;	// Right side, 2nd set

	double middle_radius;	// Average of outer and inner radius

	// Draw outer, then inner circle
	GP142_circleXY(BLACK, x, y, PEACE_OUTER_WIDTH);
	GP142_circleXY(BGColor, x, y, PEACE_INNER_WIDTH);

	// Find radius between larger and smaller radius
	middle_radius = (PEACE_OUTER_WIDTH + PEACE_INNER_WIDTH) / 2;

	// Get x,y coordinates to draw peace sign
	x1 = (int) (COS225 * middle_radius) + x;
	y1 = (int) (SIN225 * middle_radius) + y;

	x2 = (int) (COS315 * middle_radius) + x;
	y2 = (int) (SIN315 * middle_radius) + y;

	x3 = x;
	y3 = (int) middle_radius + y;

	// Draw bars to middle
	GP142_lineXY(BLACK, x1, y1, x, y, PEACE_BAR_WIDTH);
	GP142_lineXY(BLACK, x2, y2, x, y, PEACE_BAR_WIDTH);
	GP142_lineXY(BLACK, x3, y3, x, y, PEACE_BAR_WIDTH+1);

	// Draw 3 bands
	for (t=0; t <3; t++) {
		// Calculate inner point offest for running bands effect:
		// x will be negative
		left_x1 = (int) (cos( ((*frame + 90) /180.0) * PI) * 
			(PEACE_INNER_WIDTH + 1));
		right_x1 = -(left_x1) + x;
		left_x1 = left_x1 + x;
		left_y1 = (int) (sin( ((*frame + 90) /180.0) * PI) * 
			(PEACE_INNER_WIDTH + 1)) + y;
		right_y1 = left_y1;

		// Calculate outer point offset for running bands effect:
		// x will be negative
		left_x2 = (int) (cos( ((*frame + 90) /180.0) * PI) * 
			(PEACE_OUTER_WIDTH+1));
		right_x2 = -(left_x2) + x;
		left_x2 = x + left_x2;
		left_y2 = (int) (sin( ((*frame + 90) /180.0) * PI) * 
			(PEACE_OUTER_WIDTH+1)) + y;
		right_y2 = left_y2;
	
		GP142_lineXY(color, left_x1, left_y1, left_x2, left_y2, 1);
		GP142_lineXY(color, right_x1, right_y1, right_x2, right_y2, 1);

		*frame = *frame + PEACE_RUN_INCREMENT;
		if (*frame > 360) 
			*frame = 0;
	}

} // END DrawPeace()


void DrawSmiley(int x, int y, int color, int *frame) {

	double t;			// loop var
	int smile_radius;	// radius of smile
	int x1, y1;			// smile coordinates

	GP142_circleXY(BLACK, x, y, SMILEY_WIDTH+1);
	GP142_circleXY(color, x, y, SMILEY_WIDTH);
	smile_radius = (int) (SMILEY_WIDTH * 0.75);
	
	// Draw mouth
	for (t=180; t < 360;) {
		t = t + 1;
		x1 = (int) (cos( (t/180.0) * PI) * smile_radius) + x;
		y1 = (int) (sin( (t/180.0) * PI) * smile_radius) + y;
		GP142_pixelXY(BLACK, x1, y1);
	}

	// Draw tips of mouth
	GP142_lineXY(BLACK, x - smile_radius - 2, y, x - smile_radius + 2, y, 1);
	GP142_lineXY(BLACK, x + smile_radius - 2, y, x + smile_radius + 2, y, 1);

	if (*frame != 8) {
		// Draw Eyes Open
		GP142_rectangleXY(BLACK, x - (int) (smile_radius/2), y, x - 
			(int) (smile_radius/2) + 3, y + (smile_radius / 2), FILL);
		GP142_rectangleXY(BLACK, x + (int) (smile_radius/2), y, x + 
			(int) (smile_radius/2) - 3, y + (smile_radius / 2), FILL);
	} else {
		// Draw Eyes Closed
		GP142_rectangleXY(BLACK, x - (int) (smile_radius/2) - 2, y + 7, 
			x - (int) (smile_radius/2) + 5, y + 9, FILL);
		GP142_rectangleXY(BLACK, x + (int) (smile_radius/2) + 2, y + 7, 
			x + (int) (smile_radius/2) - 5, y + 9, FILL);
	}

	*frame = *frame + 1;	// Increment frame

	// Reset frame if the face blinks
	if (*frame == TIME_TO_BLINK) 
		*frame = 1;

} // END DrawSmiley()

void FindObject(int x, int y, int ShapeArray[ARRAYSIZE][5], int numshapes, 
				int *currentShape, int *hilight) {

	int t;			// Loop var
	int x1, y1;		// Object coordinates
	double t_dist;	// Total Distance
	int x_dist;		// X-Distance
	int y_dist;		// Y-Distance
	double shortest_d;	// Possible wanted coordinates

	shortest_d = 100;	// Give big default value, so first good coordinate
						// is always smaller.

	// Compare mouse values clicked with all possible array coordinates, and
	// see which one is the closest match. But must be within 10 pixels
	for (t=0; t < numshapes; t++) {
		x1 = ShapeArray[t][0];
		y1 = ShapeArray[t][1];

		x_dist = x1 - x;
		y_dist = y1 - y;
		t_dist = sqrt((x_dist * x_dist) + (y_dist * y_dist));

		printf("%f, %d", t_dist, t);

		if (t_dist < ERROR_MARGIN && t_dist < shortest_d) {
			shortest_d = t_dist;
			*currentShape = t;
			switch (ShapeArray[t][4]) {
			case HEART:
				*hilight = SEL_HEART;
				break;

			case MESSAGE:
				*hilight = SEL_MESSAGE;
				break;

			case PEACE:
				*hilight = SEL_PEACE;
				break;

			case SMILEY:
				*hilight = SEL_SMILEY;
				break;

			default:
				break;
			}
		}
	}

} // END FindObject()



int GetColor (int color) {
// Only allow bright palletes! ;)

	switch (color) {
	case RED:
		return(GREEN);
		break;

	case GREEN:
		return(YELLOW);
		break;

	case YELLOW:
		return (MAGENTA);
		break;

	case MAGENTA:
		return (TURQUOISE);
		break;

	case TURQUOISE:
		return (ORANGE);
		break;

	case ORANGE:
		return (PINK);
		break;

	case PINK:
		return (CHALK);
		break;

	case CHALK:
		return (PEACH);
		break;

	case PEACH:
		return (SEA_GREEN);
		break;

	case SEA_GREEN:
		return (RED);
		break;

	default:
		return (color);
	}
} // End GetColor()


void HandleAction(int *x, int *y, int ButtonStatus, int *hilight, 
				  int ShapeArray[ARRAYSIZE][5], int *ns, int *BGColor, 
				  int *MentalStatus, int * currentShape, char *key_hit) {
	// Note: *NS = Number of shapes

	int t;		// Loop variable
	int t2;		// Loop variable

	// user clicked on something
	switch (ButtonStatus) {

	case SEL_COLOR: // user clicked on color

		// check which box was highlighted, and proceed
		switch (*hilight) {

			case SEL_BACKGROUND:
				// Change background color
				*BGColor = GetColor(*BGColor);
				break;

			case SEL_HEART:
				// If there is a current object selected, then
				// change the shape color
				if (*ns != NOTHING)
					ShapeArray[*currentShape][2] = 
					GetColor(ShapeArray[*currentShape][2]);
				break;

			case SEL_SMILEY:
				if (*ns != NOTHING)
					ShapeArray[*currentShape][2] = 
					GetColor(ShapeArray[*currentShape][2]);
				break;

			case SEL_PEACE:
				if (*ns != NOTHING)
					ShapeArray[*currentShape][2] = 
					GetColor(ShapeArray[*currentShape][2]);
				break;

			case SEL_MESSAGE:
				if (*ns != NOTHING)
					ShapeArray[*currentShape][2] = 
					GetColor(ShapeArray[*currentShape][2]);
				break;

			default:
				break;
		} // END switch (hilight)
		break;

	case SEL_WISDOM:
		Wisdom(&*x, &*y, &*key_hit);
		break;

	case SEL_HIT:
		if (*MentalStatus == HIT) 
			*MentalStatus = NORMAL;
		else
			*MentalStatus = HIT;
		break;

	case SEL_TRIP:
		if (*MentalStatus == TRIP) 
			*MentalStatus = NORMAL;
		else
			*MentalStatus = TRIP;
		break;

	case SEL_CLEAR:
		for (t=0; t < ARRAYSIZE; t++) {
			for (t2=0; t2 < 5; t2++) 
				ShapeArray[t][t2] = 0;
		}
		break;

	case SEL_CARD:
		switch (*hilight) {

			case SEL_HEART: // Add a heart
				 // Make sure we stay in array
				if (*ns < ARRAYSIZE) {
					ShapeArray[*ns][0] = *x;
					ShapeArray[*ns][1] = *y;
					ShapeArray[*ns][2] = DEFAULT_HEART_COLOR;
					ShapeArray[*ns][3] = 1;		// starting frame
					ShapeArray[*ns][4] = HEART;
					*currentShape = *ns;
					*ns = *ns + 1;
				}
				break;

			case SEL_SMILEY:
				// Make sure we stay in array
				if (*ns < ARRAYSIZE) {
					ShapeArray[*ns][0] = *x;
					ShapeArray[*ns][1] = *y;
					ShapeArray[*ns][2] = DEFAULT_SMILEY_COLOR;
					ShapeArray[*ns][3] = 1;		// starting frame
					ShapeArray[*ns][4] = SMILEY;
					*currentShape = *ns;
					*ns = *ns + 1;
				}
				break;

			case SEL_PEACE:
				// Make sure we stay in array
				if (*ns < ARRAYSIZE) {
					ShapeArray[*ns][0] = *x;
					ShapeArray[*ns][1] = *y;
					ShapeArray[*ns][2] = DEFAULT_PEACE_COLOR;
					ShapeArray[*ns][3] = 1;		// starting frame
					ShapeArray[*ns][4] = PEACE;
					*currentShape = *ns;
					*ns = *ns + 1;
				}
				break;

			case SEL_MESSAGE:
				// Make sure we stay in array
				if (*ns < ARRAYSIZE) {
					// Send a random frame value in
					t = rand() % 7;
					ShapeArray[*ns][0] = *x;
					ShapeArray[*ns][1] = *y;
					ShapeArray[*ns][2] = DEFAULT_MSG_COLOR;
					ShapeArray[*ns][3] = t;		// starting frame
					ShapeArray[*ns][4] = MESSAGE;
					*currentShape = *ns;
					*ns = *ns + 1;
				}
				break;

			case SEL_OBJECT:
				FindObject(*x, *y, ShapeArray, *ns, &*currentShape, &*hilight);
				break;

			default:
				break;
		}

		break;

	default:
		break;
	} // END switch

} // END HandleAction()


int MouseClick(int mouse_x, int mouse_y) {

	int t;				// Loop var
	int numNull = 2;	// Number of areas w/o buttons
	int x1, y1, x2, y2;	// Box Dimensions

	// Array of areas
	int bArea[MAX_BUTTONS+2] = {
		SEL_BACKGROUND,
		SEL_HEART,
		SEL_SMILEY,
		SEL_PEACE,
		SEL_MESSAGE,
		SEL_OBJECT,
		SEL_COLOR,
		SEL_NONE,
		SEL_HIT,
		SEL_TRIP,
		SEL_WISDOM,
		SEL_NONE,
		SEL_CLEAR};

	x1 = XMIN + BUTTON_XPAD;
	x2 = x1 + BUTTON_WIDTH;

	// Check for possible button click if within X dimensions
	if (mouse_x >= x1 && mouse_x <= x2) {
	  	for(t=0; t < (MAX_BUTTONS + numNull) ; t++) {	
	
			// Find all the possibly Y ranges for the buttons
			y1 = (YMAX - BUTTON_YPAD) - (BUTTON_HEIGHT * t);
			y2 = y1 - BUTTON_HEIGHT;

			// Check if click in a valid y-range, and
			// if so, return the button clicked
			if (mouse_y <= y1 && mouse_y >= y2)
				return bArea[t];
		}
	} else
		// If the user clicked on the card
		if (mouse_x >= CARD_X1 && mouse_x <= CARD_X2 && 
			mouse_y >= CARD_Y1 && mouse_y <= CARD_Y2)
			return SEL_CARD;
	else
		// Nothing clicked
		return SEL_NONE;
	
	return SEL_NONE;

} // END ClassifyMouseClick()


// *************  Just extra neat fuctions after this point  **************
// *************    Intro, special buttons, ending, etc..    **************

void TripOutTime(int ShapeArray[ARRAYSIZE][5], int numShapes, int *x, int *y, 
				 char *key, int *quit, int *MentalStatus) {
// User goes on an interesting trip..

	int i;					// loop variables
	int t, t2;				// loop variables
	int event;				// used for GP 142 event
	int out_rad;			// used in random lines
	int in_rad;				// used in random lines
	int smiley_angle;		// angle of rotation
	int smiley_color;		// color of the smile guy
	double smiley_increment;// amount in increment smiley
	int smiley_mouth;		// size of the smiley mouth
	int smiley_radius;		// radius of smile's head
	int smiley_x;			// x coordinate of head
	int smiley_y;			// y coordinate of head
	int radius;				// radius of spiral
	int maxrad;				// max spiral radius
	int tunneldone;			// if the tunnel has finished drawing
	double incr_radius;		// amount of increment

	int x1, x2, x3, x4;		// coordinates
	int y1, y2 ,y3, y4;

	tunneldone = FALSE;				// tunnel has not been drawn yet
	smiley_radius = 5;				// default head size
	smiley_increment = .3;			// amount to increment
	smiley_color = YELLOW;			// set it default to yellow
	smiley_angle = 0;				// start out at the top
	
	radius = 1;						// initial radius
	incr_radius = 1;				// amount to increment
	maxrad = (int) (XMAX * 1.6);	// Just outside of the screen

	// While user hasn't hit any buttons/clicks
	while (!*quit) {
		event = GP142_await_event(&*x, &*y, &*key);

		switch (event) {
			case GP142_QUIT:
				*quit = TRUE;	// Set flag to break loop
				return;
				break;
			
			case GP142_KBD:		// Keypress
				*MentalStatus = NORMAL;
				return;
				break;
			
			case GP142_MOUSE:	// Mouse Click
				*MentalStatus = NORMAL;
				return;
				break;
			
			default:
				break;
		} // END switch


		// T is the angle, and the amount of increase the angles
		// go everytime a circle is drawn. This makes the spiraling
		// effect.
		t = t + 3;
		if (t > 360) 
			t = t - 360;

		// A total of 40 degrees is being drawn, so repeat
		// 9 times to complete an entire circle
		for (i=0; i < 9; i++) {

			// Draw a 20 degree black pie
			x1 = (int) (cos((t/180.0) * PI) * radius);
			y1 = (int) (sin((t/180.0) * PI) * radius);
			t = t + 20;
			x2 = (int) (cos((t/180.0) * PI) * radius);
			y2 = (int) (sin((t/180.0) * PI) * radius);
			GP142_triangleXY(BLACK, x1, y1, x2, y2, 0, 0, FILL);

			// Draw a 20 degree while pie
			x1 = (int) (cos((t/180.0) * PI) * radius);
			y1 = (int) (sin((t/180.0) * PI) * radius);
			t = t + 20;
			x2 = (int) (cos((t/180.0) * PI) * radius);
			y2 = (int) (sin((t/180.0) * PI) * radius);

			GP142_triangleXY(WHITE, x1, y1, x2, y2, 0, 0, FILL);
		} // END for

		// We increate the amount by which we increment the radius,
		// thus, it's accelerating growth, so it appears to be coming
		// toward the user faster.
		radius = radius + (int) incr_radius;

		// If the radius is a certain amount, don't make it grow anymore,
		// so we don't use unnecessary CPU power.
		if (radius < maxrad)
			incr_radius = incr_radius + .2;
		else {
			incr_radius = 0;
			tunneldone = TRUE;
		}

		// Now, create some random line effects, to appear as if
		// going through the tunnel
		for (t2 = 0; t2 < 20; t2++) {

			out_rad = (rand() % radius) + 1;

			// Root it, so it's proportional to the distance from
			// the origin
			in_rad = out_rad - (int) (sqrt(out_rad) * 2);

			i = (rand() % 360) + 1;
			x3 = (int) (cos((i/180.0) * PI) * in_rad);
			y3 = (int) (sin((i/180.0) * PI) * in_rad);
			x4 = (int) (cos((i/180.0) * PI) * out_rad);
			y4 = (int) (sin((i/180.0) * PI) * out_rad);
			GP142_lineXY(MED_GRAY, x3, y3, x4, y4, 1);
		} 

		if (tunneldone == TRUE) {
			// Figure out the radius of the head, and how much it will
			// be increasing
			smiley_radius = smiley_radius + (int) smiley_increment;
			smiley_increment = smiley_increment * 1.5;

			// If the smiley face takes up the screen, make it disappear
			// and go back
			if (smiley_radius > maxrad) {
				smiley_increment = .3;
				smiley_radius = 5;
			}

			// Get the increasing angle amount, that will be used to draw
			// the coordinates of the head
			smiley_angle = smiley_angle - 3;
			if (smiley_angle < -360) 
				smiley_angle = 0;

			// Get the X and Y coordinates to draw the head at
			smiley_x = (int) (cos((smiley_angle/180.0) * PI) * 
				(smiley_radius / 2));
			smiley_y = (int) (sin((smiley_angle/180.0) * PI) * 
				(smiley_radius / 2));

			// Draw the head
			GP142_circleXY(BLACK, smiley_x, smiley_y, smiley_radius +1);
			GP142_circleXY(smiley_color, smiley_x, smiley_y, smiley_radius);
	
			// Draw mouth: Make it proportional to the size of the radius
			// This way, it'll get bigger as it gets closer to us
			smiley_mouth = (int) (smiley_radius * smiley_radius / maxrad);
			GP142_circleXY(BLACK, smiley_x, smiley_y-(smiley_radius/2) + 
				(smiley_mouth/2), smiley_mouth);

			// Draw Eyes: Right, then left.
			// Sorry that this is so garbled and entirely not understandable,
			// but I only needed many of these values twice, so I didn't 
			// stick them into variables like I probably should have. Anyhow, 
			// values of the eyes are proportionate to the size of the mouth 
			// and head.
			GP142_lineXY(BLACK, smiley_x + (int) (smiley_radius / 4), smiley_y + 
				(int) (smiley_radius / 5) + (smiley_mouth), 
				smiley_x + (int) (smiley_radius / 4), smiley_y + 
				(int) (smiley_radius / 3) + (int) (smiley_mouth / 2), 
				(int) (smiley_radius / 5));

			GP142_lineXY(BLACK, smiley_x - (int) (smiley_radius / 4), smiley_y + 
				(int) (smiley_radius / 5) + (smiley_mouth), 
				smiley_x - (int) (smiley_radius / 4), smiley_y + 
				(int) (smiley_radius / 3) + (int) (smiley_mouth / 2), 
				(int) (smiley_radius / 5));

		} // END if (tunneldone == true)

	} // END while (!quit)

} // END TripOutTime



void Wisdom(int *x, int *y, char *key_hit) {

// Draws a nice message on the screen. 

	int frame;			// which animation is on
	int event;			// GP142 event
	int smile_radius;	// radius of smile
	int t;				// Loop variables
	int quit=FALSE;		// Boolean flag
	int x1, y1;			// coordinates of smile
	int r1, r2;			// coordinates of random thing

	frame = 0;

	// event loop
	while (!quit) {

		// Draw black screen
		GP142_rectangleXY(BLACK, XMIN, YMIN, XMAX, YMAX, FILL);

		GP142_circleXY(BLACK, 0, 0, 200+1);
		GP142_circleXY(YELLOW, 0, 0, 200);
		smile_radius = 150;
	
		// Draw mouth
		for (t=180; t < 360;) {
			t = t + 1;
			x1 = (int) (cos( (t/180.0) * PI) * smile_radius);
			y1 = (int) (sin( (t/180.0) * PI) * smile_radius);
			GP142_pixelXY(BLACK, x1, y1);
		}

		// Draw tips of mouth
		GP142_lineXY(BLACK, -smile_radius - 2, 0, -smile_radius + 2, 0, 1);
		GP142_lineXY(BLACK, smile_radius - 2, 0, smile_radius + 2, 0, 1);


		// Draw Eyes Open
		frame = frame + 1;

		if (frame == 10) {
			r1 = rand() % (XMAX * 2);
			if (r1 > XMAX)
				r1 = XMAX - r1;

			r2 = rand() % (YMAX * 2);
			if (r2 > YMAX)
				r2 = YMAX - r2;
		}

		if (frame >= 11)
			GP142_printfXY(RED, r1, r2, 18, "ZoT!");


		if (frame == 20) {
			GP142_rectangleXY(BLACK, -(int) (smile_radius/2) - 10, 10, 
				-(int) (smile_radius/2) + 13, 13, FILL);
			GP142_rectangleXY(BLACK, (int) (smile_radius/2) - 10, 10, 
				(int) (smile_radius/2) + 13, 13, FILL);
			frame = 0;
		} else {
			GP142_rectangleXY(BLACK, -(int) (smile_radius/2), 0, 
				-(int) (smile_radius/2) + 3, (smile_radius / 2), FILL);
			GP142_rectangleXY(BLACK, (int) (smile_radius/2), 0, 
				(int) (smile_radius/2) - 3, (smile_radius / 2), FILL);
		}

		GP142_printfXY(WHITE, XMIN + 150, YMAX - 30, 16, 
			"\"Sometimes.. you just don't even want to ask why.\"");
	
		// Get an event 
		event = GP142_await_event(&*x, &*y, &*key_hit);
		switch (event) {
			case GP142_QUIT:	// Back to menu first
				quit = TRUE;
				return;
				break;
				
			case GP142_KBD:		// Keypress
				quit = TRUE;
				return;
				break;
				
			case GP142_MOUSE:	// Mouse Click
				quit = TRUE;
				return;
				break;

			default:
				break;
		} // END switch

	} // END while

} // END Wisdom()


void Intro (int *x, int *y, char *key_hit) {
// Perform intro sequence to starting program


	int event;				// GP142 Event
	int foo=1;				// Loop until we break
	int f1, f2, f3;			// Diffrent flags in routine

	// Used in flag1 section
	int x1, y1;

	// Used in flag2 section
	int flash, rx, ry, t, t2, t3, x2, y2;
	int p2x, p2y, p3x, p3y;
	int beamright, beamcolor, fontsize, fontup;
	int t_count, fontcolor;
	double beam_increase;
	double xrad, yrad;

	// Initializers
	f1 = TRUE; 
	x1 = 0; 
	y1 = YMIN;

	flash = TRUE; 
	t = 0; 
	xrad = 0;
	yrad = 0;

	p2x = - 8;
	p2y = YMAX;
	p3y = YMAX;
	beamright = TRUE;

	f3 = FALSE;
	beamcolor = RED;
	beam_increase = 1;
	fontsize = 32;
	fontup = FALSE;
	t_count = 0;
	fontcolor = RED;

	// We won't ever break out of this loop
	while (foo) {

		// Draw black screen
		GP142_rectangleXY(BLACK, XMIN, YMIN, XMAX, YMAX, FILL);


		// Bring up the ball, until it hits the middle of
		// the screen : Flag1 Phase
		if (f1 == TRUE) {
			GP142_circleXY(MED_GRAY, x1, y1, 6);
			GP142_circleXY(LT_GRAY, x1, y1, 4);
			GP142_circleXY(WHITE, x1, y1, 2);
			y1 = y1 + 3;

			// If it hits the middle, set flag2, kill flag1
			if (y1 >=0) {
				f1 = FALSE;
				f2 = TRUE;
			}
		}

		// Flag2 phase, make the ball begin to pulse
		if (f2 == TRUE) {
			if (flash == TRUE) {
				GP142_circleXY(WHITE, x1, y1, 6);
				GP142_circleXY(MED_GRAY, x1, y1, 4);
				GP142_circleXY(LT_GRAY, x1, y1, 2);
				flash = FALSE;
				t++;
			} else {
				GP142_circleXY(MED_GRAY, x1, y1, 6);
				GP142_circleXY(LT_GRAY, x1, y1, 4);
				GP142_circleXY(WHITE, x1, y1, 2);
				flash = TRUE;
				t++;
				}

			// t is how long we wait for the ball to pulse before
			// making the ball spark
			if ( t > 50) {

				// how many spark lines to draw loop
				for (t2 = 0; t2 < 10; t2++) {

					// get some random numbers for use as
					// radiuses. also try to get negative
					// numbers to make it more circlish
					rx = (rand() % 30) + (int) xrad;
					if (xrad < 30)
						xrad = xrad + .1;
					if (rx > 20) rx = rx - 40;
					ry = (rand() % 20) + (int) yrad;
					if (yrad < 15)
						yrad = yrad + .05;

					// draw sparks
					for (t3 = 0; t3 < 90; t3 = t3 + 2) {
						x2 = (int) (cos(t3) * ry);
						y2 = (int) (sin(t3) * rx);
						GP142_pixelXY(YELLOW, x2, y2);

						t3 = t3 + 5;
						x2 = (int) (cos(t3) * ry);
						y2 = (int) (sin(t3) * rx);
						GP142_pixelXY(OLIVE, x2, y2);
					} // END for(t3)

				}  // END for(t2)

			} // END if (t > 50)

			// draw beam coming out of ball, only after a certain interval t
			if (t > 70 && f3 == FALSE) {
				GP142_triangleXY(MED_GRAY, 0,0, p2x-8, p2y, p2x + 28, 
					p3y, FILL);
				GP142_triangleXY(NAVY_BLUE, 0,0, p2x-4, p2y, p2x + 24, 
					p3y, FILL);
				GP142_triangleXY(LT_GRAY, 0,0, p2x, p2y, p2x + 16, p3y, FILL);

				// Have the beam move to the right, unless it reaches
				// a certain point, then have it move left
				if (beamright == TRUE) {
					p2x++;
					if (p2x >= 20)
						beamright = FALSE;
				} else {
					p2x--;

					// Once it begins moving right again, have it
					// set some flags, so that the beam opens up
					if (p2x <= -20) {
						beamright = TRUE;
						f3 = TRUE;
						p3x = p2x + 16;
					}
				}
			}

			// Flag3 phase, beam opens up, and reveals message
			if (f3 == TRUE) {
				beamcolor = GetColor(beamcolor);
				GP142_triangleXY(MED_GRAY, 0,0, p2x-8, p2y, p3x + 8, p3y, FILL);
				GP142_triangleXY(NAVY_BLUE, 0,0, p2x-4, p2y, p3x + 4, p3y, FILL);
				GP142_triangleXY(beamcolor, 0,0, p2x, p2y, p3x, p3y, FILL);

				// increase the x-offset of the beam in both directions
				p2x = p2x - (int) beam_increase;
				p3x = p3x + (int) beam_increase;
				if (p2x > (XMIN + 50))
					beam_increase = beam_increase + .5;
				else {
					// once the beam has stopped, wait for 60 events before
					// throwing user into main
					beam_increase = 0;
					t_count = t_count + 1;
					if (t_count == 90)
						return;

				}
			}

			// If about to end, draw a white screen
			if (t_count > 50)
				GP142_rectangleXY(WHITE, XMIN, YMIN, XMAX, YMAX, FILL);

			// draw order so that the messages are last, so they are
			// sillouetted
			if (t_count < 50) {
				GP142_printfXY(BLACK, -70, YMAX - 40, fontsize, "The Trippy");
				GP142_printfXY(BLACK, -100, YMAX - 80, fontsize-1, "Valentine's Day");
				GP142_printfXY(BLACK, -40, YMAX - 120, fontsize-2, "Card");

			// But if the screen is white, make it colorful
			} else {
				fontcolor = GetColor(fontcolor);
				GP142_printfXY(fontcolor, -70, YMAX - 40, fontsize, "The Trippy");
				fontcolor = GetColor(fontcolor);
				GP142_printfXY(fontcolor, -100, YMAX - 80, fontsize-1, "Valentine's Day");
				fontcolor = GetColor(fontcolor);
				GP142_printfXY(fontcolor, -40, YMAX - 120, fontsize-2, "Card");
			}

			// once the beam has stopped also, make the letters jump around
			// a little
			if (t_count != 0) {
				if (fontup == FALSE)
					fontsize = fontsize - 1;
				else
					fontsize = fontsize + 1;

				if (fontsize < 30)
					fontup = TRUE;
				if (fontsize > 35)
					fontup = FALSE;
			}
			
		} // END if (f2 == TRUE)


		event = GP142_await_event(&*x, &*y, &*key_hit);
		
		// Run the intro unless the user hits a key
		// main program
		switch (event) {
			case GP142_QUIT:	// Back to menu first
				return;
				break;
			case GP142_KBD:		// Keypress
				return;
				break;
			case GP142_MOUSE:	// Mouse Click
				return;
				break;
			default:
				break;
		} // END switch

	} // END while

} // END Intro()


void Ending (int *x, int *y, char *key_hit) {
// Don't suppose we could mess with the event timer now could we?...


	int pointarray[148][2] = {

{10, 256}, 
{12, 256}, 
{21, 252}, 
{27, 249}, 
{34, 244}, 
{42, 238}, 
{51, 231}, 
{65, 215}, 
{72, 204}, 
{79, 190}, 
{87, 166}, 
{88, 152}, 
{87, 150}, 
{85, 152}, 
{80, 160}, 
{81, 168}, 
{85, 177}, 
{89, 189}, 
{97, 209}, 
{98, 219}, 
{93, 237}, 
{76, 251}, 
{58, 252}, 
{29, 238}, 
{18, 228}, 
{18, 228}, 
{32, 234}, 
{52, 240}, 
{82, 245}, 
{92, 244}, 
{118, 235}, 
{128, 226}, 
{132, 218}, 
{130, 214}, 
{127, 216}, 
{120, 220}, 
{107, 235}, 
{107, 244}, 
{112, 250}, 
{130, 252}, 
{157, 239}, 
{168, 221}, 
{168, 216}, 
{165, 213}, 
{162, 212}, 
{148, 224}, 
{144, 232}, 
{141, 240}, 
{141, 246}, 
{145, 250}, 
{166, 252}, 
{172, 250}, 
{174, 250}, 
{271, 166}, 
{267, 169}, 
{263, 171}, 
{256, 172}, 
{251, 169}, 
{251, 162}, 
{255, 156}, 
{261, 150}, 
{268, 148}, 
{276, 147}, 
{281, 152}, 
{281, 165}, 
{280, 176}, 
{278, 186}, 
{275, 197}, 
{271, 207}, 
{264, 224}, 
{262, 231}, 
{262, 237}, 
{261, 244}, 
{261, 246}, 
{262, 246}, 
{272, 242}, 
{289, 227}, 
{296, 220}, 
{298, 217}, 
{299, 217}, 
{297, 226}, 
{291, 245}, 
{285, 258}, 
{271, 281}, 
{263, 291}, 
{250, 308}, 
{240, 316}, 
{232, 323}, 
{228, 326}, 
{226, 327}, 
{229, 318}, 
{251, 289}, 
{274, 270}, 
{297, 251}, 
{316, 235}, 
{324, 228}, 
{334, 221}, 
{343, 215}, 
{352, 210}, 
{356, 209}, 
{359, 207}, 
{360, 207}, 
{343, 214}, 
{333, 220}, 
{325, 232}, 
{318, 243}, 
{315, 252}, 
{316, 252}, 
{333, 242}, 
{342, 234}, 
{353, 215}, 
{357, 208}, 
{358, 207}, 
{358, 208}, 
{355, 216}, 
{350, 226}, 
{346, 242}, 
{346, 246}, 
{347, 246}, 
{361, 245}, 
{361, 245}, 
{418, 231}, 
{415, 234}, 
{413, 238}, 
{412, 240}, 
{416, 240}, 
{419, 237}, 
{420, 236}, 
{420, 234}, 
{419, 234}, 
{421, 208}, 
{421, 207}, 
{424, 191}, 
{426, 181}, 
{430, 163}, 
{432, 154}, 
{436, 139}, 
{439, 130}, 
{442, 122}, 
{442, 122}, 
{441, 130}, 
{437, 148}, 
{431, 172}, 
{429, 181}, 
{423, 200}, 
{420, 207}, 
{419, 208}, 
{419, 208}
 };


	int t, t2;		// Loop var
	int x1, y1;		// Coordinate set 1
	int x2, y2;		// Coordinate set 2
	int color;		// Line Color
	int distance;	// distance between points
	int event;		// GP event

	// Draw black screen
	GP142_rectangleXY(BLACK, XMIN, YMIN, XMAX, YMAX, FILL);

	x1 = 0; y1 = 0; x2 = 0; y2 = 0;
	color = RED;

	// Loop through the entire array, connecting one set to the
	// next with lines

	for (t=0; t < 148; ) {
		for (t2 = 0; t2 < 2; t2++) {
			x2 = pointarray[t][0] + XMIN;
			y2 = YMAX - pointarray[t][1];

			// For the first time, they need to start off
			// at the same point
			if (t == 0) {
				x1 = x2;
				y1 = y2;
			}

			// Only connect the dots that are relatively close together,
			// otherwise, lines will go from one word to the next
			distance = (int) (sqrt((x1-x2)*(x1-x2) + (y1-y2)*(y1-y2)));

			if (distance < 40) {
				//GP142_lineXY(MED_GRAY, x1, y1, x2, y2, 5);
				//GP142_lineXY(LT_GRAY, x1, y1, x2, y2, 3);
				GP142_lineXY(WHITE, x1, y1, x2, y2, 3);
				//GP142_rectangleXY(RED, x1, y1, x2, y2, 1);
			}

			x1 = x2;
			y1 = y2;

			t++;

		} // END for(t2)

		// Get an event 
		event = GP142_await_event(&*x, &*y, &*key_hit);
		
		// Don't quit straight of if the x is hit, but go back to the
		// main program
		switch (event) {
			case GP142_QUIT:	// Back to menu first
				return;
				break;
			
			case GP142_KBD:		// Keypress
				return;
				break;
			
			case GP142_MOUSE:	// Mouse Click
				return;
				break;

			default:
				break;
		} // END switch

	} // END for(t)

} // END Wisdom()
